-module(akh_lexer).
-include("akh_source_map.hrl").
-export([tokenize/1]).

-type token_kind() ::
    line_comment
    | block_comment
    | atom
    | identifier
    | integer
    | '{'
    | '}'
    | '['
    | ']'
    | '('
    | ')'
    | ','
    | ';'
    | '.'
    | '->'
    | '+'.

-type token() :: {token_kind(), span()}.

-spec tokenize(string()) -> {ok, source_map(), [token()]}.
tokenize(Text) -> tokenize(Text, 0, 1, akh_source_map:empty(), []).

-spec tokenize(
    string(),
    Offset :: non_neg_integer(),
    Line :: pos_integer(),
    SourceMap :: source_map(),
    [token()]
) -> {ok, source_map(), [token()]}.
tokenize([], _, _, SourceMap, Tokens) ->
    {ok, SourceMap, Tokens};
tokenize([$\n | T], Offset, Line, SourceMap, Tokens) ->
    NewMap = akh_source_map:insert(Offset, Line, SourceMap),
    tokenize(T, Offset + 1, Line + 1, NewMap, Tokens);
tokenize([_ | T], Offset, Line, SourceMap, Tokens) ->
    tokenize(T, Offset + 1, Line, SourceMap, [{atom, {Offset, 0}} | Tokens]).
